package org.this_voice.modabrowser;

import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import android.app.ListActivity;
import android.content.Intent;
import android.os.Bundle;
import android.os.Environment;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ArrayAdapter;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;
import android.view.Gravity;

public class ChooseBlog extends ListActivity {

	private final int BLOG_POST = 0;
	private final int ADDEDIT_BLOG = 1;

	private final String[] blogs = {
		"Poor Richard's Alchemy\thttp://this-voice.org/cgi-bin/blink.cgi?mode=blog&dir=Poor_Richards_Alchemy&fname=$SECONDS&type=write&title=$TITLE&content=$CONTENT&month=$MONTH",
		"Ephemera\thttp://this-voice.org/cgi-bin/fmra.cgi?mode=blog&dir=ephemera&fname=$SECONDS&type=write&content=$CONTENT&month=$MONTH"
	};

	private PRAFileHandler fhandler = new PRAFileHandler(this);
	private String bloglist = "bloglist";
	private Map<String, String> dbs = new HashMap<String, String>();
	private String sep = "\t"; // must be invalid URL char
	private String link = "";

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		Intent inc = getIntent();
		link = inc.getStringExtra("LINK");
		setContentView(R.layout.activity_chooseblog);
	}


	@Override
	public void onResume() {
		super.onResume();

		final String[] list = buildList();
		setListAdapter(new ArrayAdapter<String>(this,
				android.R.layout.simple_list_item_1, list));

		ListView listView = getListView();
		listView.setTextFilterEnabled(true);
		registerForContextMenu(listView);

		listView.setOnItemClickListener(new OnItemClickListener() {
			public void onItemClick(AdapterView<?> parent, View view,
					int position, long id) {
				String name = ((TextView) view).getText().toString();
				String url = dbs.get(name);
				post(name, url);
			}
		});
	}

	private void post(String name, String url){
		Intent i = new Intent(this, BlogPost.class);
		i.putExtra("NAME", name);
		i.putExtra("URL", url);
		i.putExtra("LINK",link);
		startActivityForResult(i, BLOG_POST);
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		if (requestCode == BLOG_POST) {
			Intent intent = new Intent();
			if (resultCode == RESULT_OK) {
				final String url = data.getStringExtra("URL");
				intent.putExtra("URL", url);
				setResult(RESULT_OK, intent); 
				finish();
			} else {
				;//change blog?
			}
		}else if (requestCode == ADDEDIT_BLOG) {
			if (resultCode == RESULT_OK) {
				String blog = data.getStringExtra("BLOG");
				fhandler.writeUniqLineByTokens(bloglist, blog, sep);
				onResume();
			} else {
				;//??
			}
		}
    }

	private String[] buildList() {
		String[] blogfile = fhandler.getFileAsArray(bloglist);
		List<String> alist = new ArrayList<String>();

		for (String s : blogfile) {
			String[] t = s.split(sep);
			dbs.put(t[0], t[1]);
			alist.add(t[0]);
		}

		for (String s : blogs) {
			String[] t = s.split(sep);
			dbs.put(t[0], t[1]);
			alist.add(t[0]);
		}

		Collections.sort(alist);
		String[] list = new String[alist.size()];
		alist.toArray(list);
		return list;
	}


	@Override
	public void onCreateContextMenu(ContextMenu menu, View v,
			ContextMenuInfo menuInfo) {
		super.onCreateContextMenu(menu, v, menuInfo);
		menu.setHeaderTitle("Context Menu");
		menu.add(0, v.getId(), 0, "Add or Edit");
		menu.add(0, v.getId(), 0, "Remove");
	}

	@Override
	public boolean onContextItemSelected(MenuItem item) {
		AdapterView.AdapterContextMenuInfo info = (AdapterView.AdapterContextMenuInfo) item
				.getMenuInfo();
		TextView view = (TextView) info.targetView;
		String name = view.getText().toString();
		String url = dbs.get(name);
     	if (item.getTitle() == "Add or Edit") {
			Intent intent = new Intent(this, AddEditBlog.class);
			intent.putExtra("NAME",name);
			intent.putExtra("URL", url);
			startActivityForResult(intent, ADDEDIT_BLOG);
            return true;
		} else if (item.getTitle() == "Remove") {
			fhandler.removeLineByToken0(bloglist, name, sep);
			onResume();
            return true;
		} else
			return false;
    }

	@Override
	public void onBackPressed() {
		Intent intent = new Intent();
		setResult(RESULT_CANCELED, intent);
		finish();
	}

}
