package org.this_voice.modabrowser;

import android.content.Context;
import android.app.AlertDialog;
import android.app.ListActivity;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.os.Environment;
import android.text.InputType;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.ContextMenu;
import android.view.Gravity;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.lang.String;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * ChooseContext.java created and copyrighted: 10 Jun 13
 *
 * author: R. Earle Harris (richard.earle.harris@gmail.com)
 *
 * The author releases this file under the GNU GPL license. See GNU's
 * COPYING file for specifics.
 **/

public class ChooseContext extends ListActivity {

	public Map<String, String> dbs = new HashMap<String, String>();
	public URLDecoder decoder = new URLDecoder();
	public PRAFileHandler fhandler = new PRAFileHandler(this);
	public String sep = "\t"; // must be invalid URL char

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_choosecontext);
	}


	@Override
	public void onResume() {
		super.onResume();

		final String[] list = buildList();
		setListAdapter(new ArrayAdapter<String>(this,
				android.R.layout.simple_list_item_1, list));

		ListView listView = getListView();
		listView.setTextFilterEnabled(true);
		registerForContextMenu(listView);

		listView.setOnItemClickListener(new OnItemClickListener() {
			public void onItemClick(AdapterView<?> parent, View view,
					int position, long id) {
				String result = ((TextView) view).getText().toString();
				String fname = dbs.get(result);
				Intent intent = new Intent();
				intent.putExtra("CHOOSE_CONTEXT", fname);
				setResult(RESULT_OK, intent); // we return filename of deck
				finish();
			}
		});
	}


	private String[] buildList() {
		String fname = "";
		String title = "";
		String[] tokens;
		File internal = getFilesDir();
		File[] files = internal.listFiles();
		List<String> alist = new ArrayList<String>();
		for (File inFile : files) {
		    fname = inFile.toString();
			tokens	= fname.split("/");
			fname = tokens[tokens.length - 1];
			if (fname.startsWith("context")){
				title = decoder.decode(fname.replace("context", ""));
				alist.add(title);
				dbs.put(title, fname);
			}
		}

		Collections.sort(alist);
		String[] list = new String[alist.size()];
		alist.toArray(list);
		return list;
	}


	@Override
	public void onCreateContextMenu(ContextMenu menu, View v,
			ContextMenuInfo menuInfo) {
		super.onCreateContextMenu(menu, v, menuInfo);
		menu.setHeaderTitle("Context Menu");
		menu.add(0, v.getId(), 0, "New");
		menu.add(0, v.getId(), 0, "Export");
		menu.add(0, v.getId(), 0, "Import");
		menu.add(0, v.getId(), 0, "Remove");
	}


	@Override
	public boolean onContextItemSelected(MenuItem item) {
		AdapterView.AdapterContextMenuInfo info = (AdapterView.AdapterContextMenuInfo) item
				.getMenuInfo();
		TextView view = (TextView) info.targetView;
		String name = view.getText().toString();
		if (item.getTitle() == "New") {
			newContext();
            return true;
     	} else if (item.getTitle() == "Export") {
			exportContext(name);
            return true;
     	} else if (item.getTitle() == "Import") {
            importContext();
            return true;
		} else if (item.getTitle() == "Remove") {
			removeContext(name);
            return true;
		} else
			return false;
    }

	private void importContext(){
		File sdcard = Environment.getExternalStorageDirectory();
		File[] files = sdcard.listFiles();
		String[] tokens;
		String src = "";
		String fname = "";

		for (File f : files ){
			src = f.toString();
			tokens	= src.split("/");
			fname = tokens[tokens.length - 1];
			if (fname.startsWith("context")){
				if (dbs.containsValue(fname)){
					getName(fname, src);
				} else {
					finishImport(fname, src);
				}
			}
		}
	}

	private void getName(String fname, String path){
		AlertDialog.Builder alert = new AlertDialog.Builder(this);

		String Dname = decoder.decode(fname.replace("context", ""));
		alert.setTitle("Duplicate Context Name");
		alert.setMessage("Rename import or overwrite existing context.");
		final EditText input = new EditText(this);
		input.setInputType(InputType.TYPE_CLASS_TEXT);
		alert.setView(input);
		input.setText(Dname);

		final String dir = path;
		final String orig = fname;
		alert.setPositiveButton("Enter", new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int whichButton) {
				String name = input.getText().toString();
				try {
					name = URLEncoder.encode(name,"UTF-8");
				} catch (UnsupportedEncodingException u) { ; }
				finishImport(name, dir);
			}
			});

		alert.setNegativeButton("Overwrite",
				new DialogInterface.OnClickListener() {
					public void onClick(DialogInterface dialog, int whichButton) {
						finishImport(orig, dir);
					}
				});
		alert.show();
	}

	// fname is new encoded context name, path is original file to import
	private void finishImport(String fname, String path){
		File sdcard = Environment.getExternalStorageDirectory();
		String[] t	= path.split("/");
		String filename = t[t.length - 1];
		String[] orig = fhandler.getExtFileAsArray(sdcard.toString()+"/", filename);
		String imported = "context"+fname;
		boolean isNode = true;
		String annot = "";
		for (String s : orig){
			if (isNode){
				fhandler.writeUniqLineByTokens(imported, s, sep);
				t = s.split(sep);
				annot = Integer.toString(t[0].hashCode());
				annot = annot.replace('-', 'm');
				annot = fname + annot;
				isNode = false;
			} else if (s.matches("")){
				isNode = true;
			} else {
				FileOutputStream f;
				try {
					boolean append = true;
					f = openFileOutput(annot, Context.MODE_APPEND);
					f.write(s.getBytes());
					f.write(System.getProperty("line.separator").getBytes());
					f.flush();
					f.close();
				} catch (Exception e){ ; }
			}
		}
		onResume();
	}

	private void newContext(){

		AlertDialog.Builder alert = new AlertDialog.Builder(this);

		alert.setTitle("New Context");
		alert.setMessage("Enter Context Name");
		final EditText input = new EditText(this);
		input.setInputType(InputType.TYPE_CLASS_TEXT);
		alert.setView(input);

		alert.setPositiveButton("Create", new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int whichButton) {
				String name = input.getText().toString();
				String test = dbs.get(name);
				if ((test == null) || (test.matches(""))){
					String fname = "context";
					try {
						fname = fname + URLEncoder.encode(name,"UTF-8");
					} catch (UnsupportedEncodingException u) { ; }
					fhandler.touchFile(fname);
					onResume();
				}
			}
		});

		alert.setNegativeButton("Cancel",
				new DialogInterface.OnClickListener() {
					public void onClick(DialogInterface dialog, int whichButton) {
						onResume();
					}
				});
		alert.show();

	}

	private void exportContext(String name){
		String fname = dbs.get(name);
		String prefix = fname.replace("context","");
		File sdcard = Environment.getExternalStorageDirectory();
		String[] file2export = fhandler.getFileAsArray(fname);
		File exportfile = new File(sdcard, fname);
		FileOutputStream writer = null;
		try {
			writer = new FileOutputStream(exportfile);
		} catch (java.io.FileNotFoundException fnf) {;}

		for (String s : file2export) {
			try {
				// write node
				writer.write(s.getBytes());
				writer.write(System.getProperty("line.separator").getBytes());
				writer.flush();

				// write annotation
				String[] t = s.split(sep);
				String annot = Integer.toString(t[0].hashCode());
				annot = annot.replace('-', 'm');
				annot = prefix + annot;
				String[] orig = fhandler.getFileAsArray(annot);
				for (String st : orig) {
					writer.write(st.getBytes());
					writer.write(System.getProperty("line.separator").getBytes());
					writer.flush();
				}
				
				// write blank line
				writer.write(System.getProperty("line.separator").getBytes());
				writer.flush();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}

		try {
			writer.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		toaster(getResources().getString(R.string.context)+" "+
				name+" "+getResources().getString(R.string.exported), Toast.LENGTH_SHORT);
	}


	private void removeContext(String name){
		final String fname = dbs.get(name);

		AlertDialog.Builder b = new AlertDialog.Builder(this);
		b.setTitle("Context Deletion");
		b.setMessage("Remove "+name+"?");
		b.setPositiveButton("Yes",new DialogInterface.OnClickListener() {
			    @Override
					public void onClick(DialogInterface dialog, int whichButton) {
					String[] t = null;
					String annot = "";
					String[] orig = fhandler.getFileAsArray(fname);
					String prefix = fname.replace("context","");
					for (String s : orig){
						t = s.split(sep);
						annot = Integer.toString(t[0].hashCode());
						annot = annot.replace('-', 'm');
						annot = prefix + annot;
						fhandler.remove(annot);
					}
                    fhandler.remove(fname);
					onResume();

				}
			});
		b.setNegativeButton("No", null);
		b.create().show();
	}


	@Override
	public void onBackPressed() {
		Intent intent = new Intent();
		setResult(RESULT_CANCELED, intent);
		finish();
	}


	public void toaster(String text, int length) {
		Toast t = Toast.makeText(this, text, length);
		t.setGravity(Gravity.CENTER, 0, 0);
		t.show();
	}

}
