package org.this_voice.modabrowser;

import android.content.Context;
import android.util.AttributeSet;
import android.view.GestureDetector;
import android.view.Gravity;
import android.view.MotionEvent;
import android.webkit.WebView;
import android.widget.Toast;

import java.util.ArrayList;
import java.util.List;

/**
 * MoDaView.java created and copyrighted: 4 April 13
 *
 * author: R. Earle Harris (richard.earle.harris@gmail.com)
 *
 * The author releases this file under the GNU GPL license. See GNU's COPYING
 * file for specifics.
 **/

public class MoDaView extends WebView implements
		GestureDetector.OnDoubleTapListener {

	public Context mContext;
	public PRAFileHandler fhandler;
	public String sep = "\t"; // must be invalid URL char
	public String[] contextList;
	public boolean contextTravel = false;
	public boolean twostep = false;
	public final MoDaView mWV = this;
	public int contextLength;
	public int current = 0;
	public MoDaBrowser MoDa;

	// deprecated usage used to let WebView handle own gestures
	@SuppressWarnings("deprecation")
	public GestureDetector gestureDetector = new GestureDetector(
			new mGestureDetector());

	public MoDaView(Context context) {
		super(context);
		mContext = context;
		fhandler = new PRAFileHandler(mContext);
		gestureDetector.setOnDoubleTapListener(this);
		gestureDetector.setIsLongpressEnabled(true);
	}

	public MoDaView(Context context, AttributeSet attrs, int defStyle) {
		super(context, attrs, defStyle);
		mContext = context;
		fhandler = new PRAFileHandler(mContext);
		gestureDetector.setOnDoubleTapListener(this);
		gestureDetector.setIsLongpressEnabled(true);
	}

	public MoDaView(Context context, AttributeSet attrs) {
		super(context, attrs);
		mContext = context;
		fhandler = new PRAFileHandler(mContext);
		gestureDetector.setOnDoubleTapListener(this);
		gestureDetector.setIsLongpressEnabled(true);
	}

	public void setMoDa(MoDaBrowser moda){
		MoDa = moda;
	}

	public void toggleTravel(String currentContext){
		reloadContext(currentContext);
		contextTravel = !contextTravel;
		String val = "";
		if (contextTravel) {
			val = getResources().getString(R.string.context);
		} else {
			val = getResources().getString(R.string.history);
		}
		Toast toast = Toast.makeText(mContext, getResources().getString(R.string.swiping)
									 +" " + val, Toast.LENGTH_SHORT);
		toast.setGravity(Gravity.TOP | Gravity.CENTER, 0, 0);
		toast.show();
	}

	public void reloadContext(String currentContext){
		List<String> tmpList = new ArrayList<String>();
		String[] file = fhandler.getFileAsArray(currentContext);
		for (String s : file) {
			String[] t = s.split(sep);
			tmpList.add(t[1]);
		}
		contextList = new String[tmpList.size()];
		tmpList.toArray(contextList);
		contextLength = contextList.length;
		MoDa.loadPage4Activity(contextList[0]);
	}

	public void contextPrev(){
		current = current - 1;
		if (current < 0) { current = contextLength - 1; }
		MoDa.loadPage4Activity(contextList[current]);
	}

	public void contextNext(){
		current = current + 1;
		if (current == contextLength ) { current = 0; }
		MoDa.loadPage4Activity(contextList[current]);
	}

	public void toggleTwoStep(){
		twostep = !twostep;
		String val = "";
		if (twostep) {
			val = getResources().getString(R.string.on);
		} else {
			val = getResources().getString(R.string.off);
		}
		Toast toast = Toast.makeText(mContext, getResources().getString(R.string.historytwostep)
									 +" " + val, Toast.LENGTH_SHORT);
		toast.setGravity(Gravity.TOP | Gravity.CENTER, 0, 0);
		toast.show();
	}

	/**
	 * GESTURES
	 */

	private static final int SWIPE_MIN_DISTANCE = 250;
	private static final int SWIPE_MAX_OFF_PATH = 200;
	private static final int SWIPE_THRESHOLD_VELOCITY = 200;

	class mGestureDetector extends GestureDetector.SimpleOnGestureListener {

		@Override
		public boolean onDown(MotionEvent e) {
			return false;
		}

		@Override
		public void onLongPress(MotionEvent e) {
			// empty
		}

		@Override
		public boolean onScroll(MotionEvent e1, MotionEvent e2,
				float distanceX, float distanceY) {
			return super.onScroll(e1, e2, distanceX, distanceY);
		}

		@Override
		public boolean onFling(MotionEvent e1, MotionEvent e2, float velocityX,
				float velocityY) {
			String msg = "";
			try {
				if (Math.abs(e1.getY() - e2.getY()) > SWIPE_MAX_OFF_PATH)
					return false;
				// right to left swipe
				if (((e1.getX() - e2.getX()) > SWIPE_MIN_DISTANCE)
						&& (Math.abs(velocityX) > SWIPE_THRESHOLD_VELOCITY)) {

					if (contextTravel){
						mWV.contextNext();
						msg = getResources().getString(R.string.nodenext);
					} else if (twostep && mWV.canGoBackOrForward(2)) {
						mWV.invalidate();
						mWV.goBackOrForward(2);
						msg = getResources().getString(R.string.historynexttwo);
					} else if (!twostep && mWV.canGoForward()) {
						mWV.invalidate();
						mWV.goForward();
						msg = getResources().getString(R.string.historynext);
					}

					// left to right swipe
				} else if (((e2.getX() - e1.getX()) > SWIPE_MIN_DISTANCE)
						&& (Math.abs(velocityX) > SWIPE_THRESHOLD_VELOCITY)) {

					if (contextTravel){
						mWV.contextPrev();
						msg = getResources().getString(R.string.nodeprev);
					} else if (twostep && mWV.canGoBackOrForward(-2)) {
						mWV.invalidate();
						mWV.goBackOrForward(-2);
						msg = getResources().getString(R.string.historyprevtwo);
					} else if (!twostep && mWV.canGoBack()) {
						mWV.invalidate();
						mWV.goBack();
						msg = getResources().getString(R.string.historyprev);
					}
				}

				if (!msg.matches("")) {
					Toast toast = Toast.makeText(mContext, msg, Toast.LENGTH_SHORT);
					toast.setGravity(Gravity.TOP | Gravity.CENTER, 0, 0);
					toast.show();
				}
				// return true; <== using this won't work!
			} catch (Exception e) {
				// nothing
			}
			return false;
		}

	}

	@Override
	public boolean onTouchEvent(MotionEvent ev) {
		// fix for documented focus bug
		switch (ev.getAction()) {
		case MotionEvent.ACTION_DOWN:
		case MotionEvent.ACTION_UP:
			if (!hasFocus()) {
				requestFocus();
			}
			break;
		}

		return gestureDetector.onTouchEvent(ev) || super.onTouchEvent(ev);
	}

	@Override
	public boolean onDoubleTap(MotionEvent event) {
		// we don't want the default zoom idiocy
		return true;
	}

	@Override
	public boolean onDoubleTapEvent(MotionEvent event) {
		// Toast.makeText(mContext, "onDoubleTapEvent",
		// Toast.LENGTH_SHORT).show();
		return false;
	}

	@Override
	public boolean onSingleTapConfirmed(MotionEvent event) {
		// Toast.makeText(mContext, "onSingleTapConfirmed", Toast.LENGTH_SHORT)
		// .show();
		return false;
	}
}
