package org.this_voice.modabrowser;

import android.app.AlertDialog;
import android.app.ListActivity;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.text.InputType;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.ContextMenu;
import android.view.Gravity;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;
import java.io.File;
import java.lang.Runnable;
import java.lang.String;
import java.net.URLDecoder;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * OpenContext.java created and copyrighted: 13 April 13
 *
 * author: R. Earle Harris (richard.earle.harris@gmail.com)
 *
 * The author releases this file under the GNU GPL license. See GNU's
 * COPYING file for specifics.
 **/

public class OpenContext extends ListActivity {

	public Map<String, String> bookmarks = new HashMap<String, String>();
	public Map<String, String> ctxts = new HashMap<String, String>();
	public PRAFileHandler fhandler = new PRAFileHandler(this);
	public String contextName = "MoDa";
	public String currentContext = "contextMoDa";
	public String sep = "\t"; // must be invalid URL char
	public String[] Bookmarks;
	public TextView title;
	public URLDecoder decoder = new URLDecoder();
	 
	public final static String firstItem = "Manage Contexts";
	public final static int CHOOSE_CONTEXT = 0;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		Intent i = getIntent();
		String newContext = i.getStringExtra("CONTEXT");
		if ((newContext != null) && (!newContext.matches(""))){
			currentContext = newContext;
			contextName = decoder.decode(currentContext.replace("context", ""));
		}
		setContentView(R.layout.activity_open_context);
		title = (TextView) findViewById(R.id.title);
	}

	@Override
	public void onResume() {
		super.onResume();
		setBookmarks();
		setListAdapter(new ArrayAdapter<String>(this, R.layout.node, Bookmarks));
		ListView listView = getListView();
		listView.setTextFilterEnabled(true);
		registerForContextMenu(listView);
		final Context context = this;
		listView.setOnItemClickListener(new OnItemClickListener() {
			@Override
			public void onItemClick(AdapterView<?> parent, View view,
					int position, long id) {
				String url;
				String title = (String) ((TextView) view).getText();
				if (title.matches(firstItem)) {
					url = firstItem;
					Intent intent = new Intent(context, ChooseContext.class);
					startActivityForResult(intent, CHOOSE_CONTEXT);
				} else {
					url = bookmarks.get(title);
					Intent intent = new Intent();
					intent.putExtra("URL", url);
					intent.putExtra("CONTEXT", currentContext);
					setResult(RESULT_OK, intent);
					finish();
				}
			}
		});
		title.setText(getResources().getString(R.string.context) + ": "+ contextName);
	}


	public void setBookmarks() {
		List<String> tmpBookmarks = new ArrayList<String>();
		String[] file = fhandler.getFileAsArray(currentContext);
		bookmarks.put(firstItem, ""); // leave here to sync array indices
		tmpBookmarks.add(firstItem);
		for (String s : file) {
			String[] t = s.split(sep);
			bookmarks.put(t[0], t[1]);
			tmpBookmarks.add(t[0]);
		}
		Bookmarks = new String[tmpBookmarks.size()];
		tmpBookmarks.toArray(Bookmarks);

	}


	@Override
	public void onCreateContextMenu(ContextMenu menu, View v,
			ContextMenuInfo menuInfo) {
		super.onCreateContextMenu(menu, v, menuInfo);
		AdapterView.AdapterContextMenuInfo info = 
			(AdapterView.AdapterContextMenuInfo) menuInfo;
		if (info.position == 0){ return; }
		menu.setHeaderTitle("Node Menu");
		menu.add(0, v.getId(), 0, "Annotate");
		menu.add(0, v.getId(), 0, "Remove");
		menu.add(0, v.getId(), 0, "Rename");
		menu.add(0, v.getId(), 0, "Share");
		menu.add(0, v.getId(), 0, "Move");
	}


	@Override
	public boolean onContextItemSelected(MenuItem item) {
		AdapterView.AdapterContextMenuInfo info = 
			(AdapterView.AdapterContextMenuInfo) item.getMenuInfo();
		final String node = Bookmarks[info.position];
		final String url = bookmarks.get(node);
		if (item.getTitle() == "Remove") {
			AlertDialog.Builder b = new AlertDialog.Builder(this);
			b.setTitle("Remove Node");
			b.setMessage(node);
			b.setPositiveButton("Remove",
					new DialogInterface.OnClickListener() {
						@Override
						public void onClick(DialogInterface dialog,
								int whichButton) {
							fhandler.removeLineByToken0(currentContext, node, sep);
							String fname = Integer.toString(node.hashCode());
							fname = fname.replace('-', 'm');
							fhandler.remove(fname);
							onResume();
						}
					});
			b.setNegativeButton("Cancel", null);
			b.create().show();
		} else if (item.getTitle() == "Rename") {
			AlertDialog.Builder b = new AlertDialog.Builder(this);
			b.setTitle("Rename Node");
			final EditText input = new EditText(this);
			input.setText(node);
			input.setInputType(InputType.TYPE_CLASS_TEXT);
			b.setView(input);
			b.setMessage("Enter new name.\nCancel aborts node.");
			b.setPositiveButton("Set Name",
					new DialogInterface.OnClickListener() {
						@Override
						public void onClick(DialogInterface dialog,
								int whichButton) {

							String newName = input.getText().toString();
							String line = newName + sep + url;
							fhandler.removeLineByToken0(currentContext, node, sep);
							fhandler.writeUniqLineByTokens(currentContext, line, sep);
							onResume();
						}
					});
			b.setNegativeButton("Cancel", null);
			b.create().show();
		} else if (item.getTitle() == "Share") {
			String fname = Integer.toString(node.hashCode());
			fname = fname.replace('-', 'm');
			String[] orig = fhandler.getFileAsArray(fname);
			String content = "";
			for (String s : orig) {
				content = content + s + "\n";
			}
			Intent emailIntent = new Intent(
					android.content.Intent.ACTION_SEND);
			emailIntent.putExtra(android.content.Intent.EXTRA_SUBJECT, node);
			emailIntent.putExtra(android.content.Intent.EXTRA_TEXT,
								 "\n\n<a href=\""+url+"\">"+node+"</a>" +  "\n\n" + 
								 content + "\n\n    ::shared from Mo Da Browser::");
			emailIntent.setType("text/plain");
			startActivity(Intent
					.createChooser(emailIntent, "Email node to ..."));
		} else if (item.getTitle() == "Annotate") {
			Intent intent = new Intent(this, Annotate.class);
			intent.putExtra("NODE", node);
			intent.putExtra("CONTEXT", currentContext);
			startActivity(intent);
		} else if (item.getTitle() == "Move") {

			String fname = "";
			String title = "";
			String[] tokens;
			File internal = getFilesDir();
			File[] files = internal.listFiles();
			List<String> alist = new ArrayList<String>();
			for (File inFile : files) {
				fname = inFile.toString();
				tokens	= fname.split("/");
				fname = tokens[tokens.length - 1];
				if ((fname.startsWith("context")) && (!fname.matches(currentContext))){
					title = decoder.decode(fname.replace("context", ""));
					alist.add(title);
					ctxts.put(title, fname);
				}
			}

			Collections.sort(alist);
			String[] list = new String[alist.size()];
			alist.toArray(list);
			final String[] listf = list;

			AlertDialog.Builder b = new AlertDialog.Builder(this);
				b.setTitle("Move "+node+" to ...")
				.setItems(listf, new DialogInterface.OnClickListener() {
               public void onClick(DialogInterface dialog, int which) {
				   String dest = ctxts.get(listf[which]);
				   String line = node + sep + url;
				   fhandler.removeLineByToken0(currentContext, node, sep);
				   fhandler.writeUniqLineByTokens(dest, line, sep);
				   onResume();
			   }


					});
				b.create().show();

		} else
			return false;
		return true;
	}


	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		if (requestCode == CHOOSE_CONTEXT) {
			if (resultCode == RESULT_OK) {
				currentContext = data.getStringExtra("CHOOSE_CONTEXT");
				contextName = decoder.decode(currentContext.replace("context", ""));
			} else {
				String[] file = fhandler.getFileAsArray(currentContext);
				if (file.length == 0){
					currentContext = "contextMoDa";
					contextName = "MoDa";
				}
			}
		}
		onResume();
    }


	@Override
	public void onBackPressed() {
		Intent intent = new Intent();
		intent.putExtra("URL", "none");
		intent.putExtra("CONTEXT", currentContext);
		setResult(RESULT_OK, intent);
		finish();
	}



    // length gets Toast.LENGTH_LONG|SHORT
	public void toaster(String text, int length) {
		Toast t = Toast.makeText(this, text, length);
		t.setGravity(Gravity.CENTER, 0, 0);
		t.show();
	}

}
