package org.this_voice.modabrowser;

import android.content.Context;
import android.content.res.AssetManager;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.Reader;
import java.io.StringWriter;
import java.io.UnsupportedEncodingException;
import java.io.Writer;
import java.lang.Runnable;
import java.lang.Thread;
import java.util.ArrayList;
import java.util.List;

/**
 * PRAFileHandler - Version 2.1
 *
 * @author riqi
 *         2.1 regex in removeLineByToken0
 *         2.0 listAssets
 *         1.9 writeArrayToExtFile,
 *         1.8 copyDirectory, deleteRecursive
 *         1.7 touchFile
 *         1.6 copyExtFile
 *         1.5 inputStreamToString
 *         1.4 getAssetAsList, getAssetAsArray
 *         1.3 copyAssets, getExtFileAsList, getExtFileAsArray
 *         1.2 simplified getFileAsArray
 *         1.1 getInternalFileAsList, writeListToFile, remove
 */


public class PRAFileHandler {

	public Context mContext;

	public PRAFileHandler(Context context) {
		mContext = context;
	}


	/**
	 *   To copy a directory: src, dst, false
	 *   To move a directory: src, dst, true; then deleteRecursive(src) 
	 **/
	public void copyDirectory(String orig, String dst, boolean deleteOrig) {
		String[] tokens	= orig.split("/");
		String dname = tokens[tokens.length - 1];
		File dest = new File(dst + "/" + dname);
		dest.mkdir();

		File src = new File(orig);
		File[] list = src.listFiles();
		for (File f : list){
			if (f.isFile()){
				copyExtFile(f.toString(), dest.toString(), deleteOrig);
			} else {
				String fs = f.toString();
				if (fs.length() > 0 && fs.charAt(fs.length()-1)=='/') {
					fs = fs.substring(0, fs.length()-1);
					f = new File(fs);
				}
				copyDirectory(f.toString(), dest.toString(), deleteOrig);
			}
		}
		
	}

    public void deleteRecursive(File path){
        if (path.isDirectory()){
            for (File f : path.listFiles()){
                deleteRecursive(f);
            }
        }
        path.delete();
    }	

	/**
	 * copies orig to dest, use deleteOrig true for mv
	 **/
	public void copyExtFile(String orig, String dst, boolean deleteOrig) {
		final String src = orig;
		final String dest = dst;
		final boolean move = deleteOrig;

		new Thread(new Runnable() {
				public void run() {

					String[] tokens	= src.split("/");
					String inputFile = tokens[tokens.length - 1];
					if (src.matches(dest +"/" + inputFile)){
						return;
					}
					
					InputStream in = null;
					OutputStream out = null;
					
					try {
						in = new FileInputStream (src);        
						out = new FileOutputStream (dest +"/" + inputFile);
						byte[] buffer = new byte[1024];
						int read;
						while ((read = in.read(buffer)) != -1) {
							out.write(buffer, 0, read);
						}
						
						in.close();
						in = null;
						out.flush();
						out.close();
						out = null;
						
						if (move){
							new File(src).delete();  
						}
					} 
					catch (FileNotFoundException fnfe1) {
						;
					}
					catch (Exception e) {
						;
					}
				}
			}).start();
		
	}//END copyExtFile


	/**
	 * @param filename
	 * @return String containing entire file
	 */
	public String inputStreamToString(String filename) {
		File file = new File(filename);
		InputStream iFile = null;
		try {
			iFile = new FileInputStream(file);
		} catch (FileNotFoundException e1) {
			// empty
		}
		Writer writer = new StringWriter();
		if (iFile != null) {
			char[] buffer = new char[1024];
			try {
				Reader reader = new BufferedReader(new InputStreamReader(
						iFile, "UTF-8"));
				int n;
				while ((n = reader.read(buffer)) != -1) {
					writer.write(buffer, 0, n);
				}
			} catch (UnsupportedEncodingException e) {
				return e.toString();
			} catch (IOException e) {
				return e.toString();
			} finally {
				try {
					iFile.close();
				} catch (IOException e) {
					return e.toString();
				}
			}
		}
		String result = writer.toString();
		return result;
	}

	/**
	 *
	 * @param fname
	 * @return fname loaded line by line into String[]
	 */
	public String[] getAssetAsArray(String fname) {
		List<String> ifile = getAssetAsList(fname);
		String[] file = new String[ifile.size()];
		ifile.toArray(file);
		return file;
	}

	/**
	 *
	 * @param fname
	 * @return fname loaded line by line into ArrayList<String>
	 */
	public List<String> getAssetAsList(String fname) {
		List<String> list = new ArrayList<String>();
		AssetManager am = mContext.getAssets();
		InputStream is;
		String line;
		try {
			is = am.open(fname);
			BufferedReader br = new BufferedReader(new InputStreamReader(
					is));
			while ((line = br.readLine()) != null) {
				list.add(line);
			}
		} catch (IOException e) {
			// empty
		}
		return list;
	}


	/**
	 * copyAssets copies all files with extension ext to target_dir uses
	 * mCopyFile
	 *
	 * @param target_dir
	 * @param ext
	 *            must include dot, ie ".txt"
	 */

	private void mCopyFile(InputStream in, OutputStream out) throws IOException {
		byte[] buffer = new byte[1024];
		int read;
		while ((read = in.read(buffer)) != -1) {
			out.write(buffer, 0, read);
		}
	}



	/**
	 * returns list of asset files with extension "ext" as String[]
	 **/
	public String[] listAssets(String ext) {
		AssetManager assetManager = mContext.getAssets();
		String[] list = null;
		try {
			list = assetManager.list("");
		} catch (IOException e) {
			; //empty
		}
		List<String> alist = new ArrayList<String>();
		for (String f : list){
			if (f.endsWith(ext)){
				alist.add(f);
			}
		}
		String[] files = new String[alist.size()];
		alist.toArray(files);

		return files;
	}



	/**
	 * copies asset files with extension "ext" to target_dir
	 * does not check for external write permissions
	 **/
	public void copyAssets(String target_dir, String ext) {
		AssetManager assetManager = mContext.getAssets();
		String[] files = null;
		try {
			files = assetManager.list("");
		} catch (IOException e) {
			// empty
		}
		for (String filename : files) {
			InputStream in = null;
			OutputStream out = null;
			if (filename.endsWith(ext)) {
				try {
					in = assetManager.open(filename);
					out = new FileOutputStream(target_dir + "/" + filename);
					mCopyFile(in, out);
					in.close();
					in = null;
					out.flush();
					out.close();
					out = null;
				} catch (IOException e) {
					// empty
				}
			}
		}
	}

	/**
	 * deletes internal fname
	 */
	public void remove(String fname) {
		mContext.deleteFile(fname);
	}

	/**
	 * Reads internal file into List<String>
	 *
	 * @param fname
	 *            Internal file to read up
	 * @return null on error, else List<String> of lines in file
	 */
	public List<String> getFileAsList(String fname) {
		List<String> ifile = new ArrayList<String>();
		String line;
		BufferedReader r = null;
		try {
			r = new BufferedReader(new InputStreamReader(
					mContext.openFileInput(fname)));
			while ((line = r.readLine()) != null) {
				ifile.add(new String(line));
			}
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			try {
				if (r != null) {
					r.close();
				}
			} catch (IOException io) {
				io.printStackTrace();
			}
		}

		try {
			if (r != null) {
				r.close();
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return ifile;
	}

	/**
	 * converts getFileAsList into String array
	 *
	 * @param fname
	 *            Internal file to read up
	 * @return String array of lines in file, empty on error
	 */
	public String[] getFileAsArray(String fname) {
		List<String> ifile = getFileAsList(fname);
		String[] file = new String[ifile.size()];
		ifile.toArray(file);
		return file;
	}

	/**
	 * Read up external file on sdcard as List
	 *
	 * @param dir
	 * @param fname
	 * @return List ifile
	 */
	public List<String> getExtFileAsList(String dir, String fname) {
		File file = new File(dir, fname);
		List<String> ifile = new ArrayList<String>();

		try {
			BufferedReader br = new BufferedReader(new FileReader(file));
			String line;

			while ((line = br.readLine()) != null) {
				if (!line.startsWith("#")) {
					ifile.add(line);
				}
			}
		} catch (IOException e) {
			// empty
		}
		return ifile;
	}

	/**
	 * converts getExtFileAsList into String array
	 *
	 * @param fname
	 *            External file to read up
	 * @return String array of lines in file, empty on error
	 */
	public String[] getExtFileAsArray(String dir, String fname) {
		List<String> ifile = getExtFileAsList(dir, fname);
		String[] file = new String[ifile.size()];
		ifile.toArray(file);
		return file;
	}


	/**
	 * @param fname is external filename
	 * @return
	 */
	public int writeArrayToExtFile(String[] lines, String fname) {
		FileOutputStream writer;
		File file;

		try {
			file = new File(fname);
            writer = new FileOutputStream(file);
		} catch (FileNotFoundException e) {
			return -1;
		}

		for (String string : lines) {
			try {
				writer.write(string.getBytes());
				writer.write(System.getProperty("line.separator").getBytes());
				writer.flush();
			} catch (IOException e) {
				return -1;
			}
		}

		try {
			writer.close();
		} catch (IOException e) {
			return -1;
		}
		return 0;
	}


	/**
	 * Removes line by unique token[0]
	 *
	 * @param fname
	 *            Internal file
	 * @param line
	 *            Line to insert
	 * @param sep
	 *            Separator of lines in fname
	 * @return -1 on error else 0
	 */
	public void removeLineByToken0(String fname, String token0, String sep) {
		String[] file = getFileAsArray(fname);
		FileOutputStream writer = null;

		try {
			writer = mContext.openFileOutput(fname, Context.MODE_PRIVATE);
		} catch (FileNotFoundException e) {
			// empty
		}

		for (String string : file) {
			if (string == null) {
				break;
			}
			String token = string.substring(0, string.indexOf(sep));
			token0 = token0.replaceAll("[^0-9A-Za-z\\s]", "");
			token = token.replaceAll("[^0-9A-Za-z\\s]", "");
			if (!token0.matches(token)) {
				try {
					writer.write(string.getBytes());
					writer.write(System.getProperty("line.separator")
							.getBytes());
					writer.flush();
				} catch (IOException e) {
					// empty
				}
			}
		}
		try {
			writer.close();
		} catch (IOException e) {
			// empty
		}
	}

	/**
	 * Inserts/updates line by unique token[0]
	 *
	 * @param fname
	 *            Internal file
	 * @param line
	 *            Line to insert
	 * @param sep
	 *            Separator of lines in fname
	 * @return -1 on error else 0
	 */
	public int writeUniqLineByTokens(String fname, String line, String sep) {
		String[] file = getFileAsArray(fname);
		FileOutputStream writer = null;

		try {
			writer = mContext.openFileOutput(fname, Context.MODE_PRIVATE);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}

		// get new data tokens
		String tmp[] = line.split(sep);
		String new0 = tmp[0];
		String new1 = tmp[1];

		for (String string : file) {
			String tmp2[] = string.split(sep);
			String old0 = tmp2[0];
			String old1 = tmp2[1];
			if ((!old1.matches(new1)) && (!old0.matches(new0))) {
				try {
					writer.write(string.getBytes());
					writer.write(System.getProperty("line.separator")
							.getBytes());
					writer.flush();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}

		// write new line of data
		try {
			writer.write(line.getBytes());
			writer.write(System.getProperty("line.separator").getBytes());
			writer.flush();
		} catch (IOException e) {
			e.printStackTrace();
		}

		try {
			writer.close();
		} catch (IOException e) {
			return -1;
		}
		return 0;
	}

	/**
	 * @param fname is internal filename
	 * @return
	 */
	public int writeListToFile(List<String> list, String fname) {
		FileOutputStream writer;

		try {
			writer = mContext.openFileOutput(fname, Context.MODE_PRIVATE);
		} catch (FileNotFoundException e) {
			return -1;
		}

		for (String string : list) {
			try {
				writer.write(string.getBytes());
				writer.write(System.getProperty("line.separator").getBytes());
				writer.flush();
			} catch (IOException e) {
				return -1;
			}
		}

		try {
			writer.close();
		} catch (IOException e) {
			return -1;
		}
		return 0;
	}


	/**
	 *  creates empty internal file "fname"
	 */
	public int touchFile(String fname) {
		FileOutputStream writer;

		try {
			writer = mContext.openFileOutput(fname, Context.MODE_PRIVATE);
		} catch (FileNotFoundException e) {
			return -1;
		}

		try {
			writer.close();
		} catch (IOException e) {
			return -1;
		}
		return 0;
	}

}
